@@ -1,5 +1,7 @@
 from .form import Form
 from .inputs import StringInput
 from .inputs import IntegerInput
-from .inputs import ChoiceInput
+from .inputs import SelectInput
+from .inputs import MultiSelectInput
 from .inputs import BooleanInput
+from .inputs import Branch
